"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNullOrUndefined = isNullOrUndefined;
exports.isObject = isObject;
exports.isPlainObject = isPlainObject;
exports.deepFreeze = deepFreeze;
function isNullOrUndefined(val) {
    return typeof val === 'undefined' || val === null;
}
function isObject(obj) {
    return !isNullOrUndefined(obj) && typeof obj === 'object';
}
function isPlainObject(obj) {
    if (typeof obj !== 'object' || obj === null || Array.isArray(obj)) {
        return false;
    }
    // 检查是否为对象字面量、`new Object()` 或从 `Object.create(null)` 创建的对象
    if (!((obj.constructor === Object || typeof obj.constructor === 'undefined')
        && (Reflect.getPrototypeOf(obj) === Object.prototype || Reflect.getPrototypeOf(obj) === null))) {
        return false;
    }
    return true;
}
function deepFreeze(obj) {
    if (!isPlainObject(obj)) {
        return;
    }
    // 不包含 Symbol 类型属性
    Object.getOwnPropertyNames(obj).forEach((name) => {
        const prop = obj[name];
        if (typeof prop === 'object' && prop !== null && !Object.isFrozen(prop)) {
            deepFreeze(prop);
        }
    });
    Object.freeze(obj);
}
//# sourceMappingURL=utils.lang.js.map